/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 10-2 - receiver.c - Receiving msgs thru window properties
 *  
 *  HDR2: Chapter 10, page 292
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include Required Header Files   ********************************/

#include <X11/Intrinsic.h>           /* The Intrinsics               */
#include <X11/Xatom.h>               /* X11 Predefined atoms         */
#include <Xm/Xm.h>                   /* Motif resource symbols       */
#include <Xm/RowColumn.h>            /* Motif XmRowColumn widget     */
#include <Xm/Label.h>                /* Motif XmLabel widget         */
#include <Xm/Text.h>                 /* Motif XmText widget          */
#include <Xm/PushB.h>                /* Motif XmPushButton widget    */

/***** Declare Global Variables  *************************************/

Display *display;
Widget toplevel, manager, label, text, exit_btn;

int    status;                       /* Return status variable       */
Atom   PRIVATE_MESSAGE,              /* Atom to identify "our" event */
       RECEIVER_WINDOW;              /* Atom to identify property    */

/***  Forward Declare Callback Routines ******************************/

void ExitRoutine();                  /* Callback - exit              */
void MessageHandler();               /* Handler for ClientMessages   */

/***  Main Program  **************************************************/

main(argc,argv)
int argc;
char **argv;
{

 XtAppContext  context; 
 Window receiver_window;                /* Top-level window ID       */

 toplevel = XtVaAppInitialize( &context,  "Receiver", 
     NULL, 0, &argc, argv, NULL, NULL );

 manager = XtCreateManagedWidget("manager",
     xmRowColumnWidgetClass, toplevel, NULL, 0);

 label = XtCreateManagedWidget("label",
     xmLabelWidgetClass, manager, NULL, 0);
 text = XtCreateManagedWidget("text",
     xmTextWidgetClass, manager, NULL, 0);

 exit_btn = XtCreateManagedWidget("exitBtn",
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( exit_btn, XmNactivateCallback,
     ExitRoutine, NULL );

 XtRealizeWidget(toplevel);             /* Realize to get windows    */

 receiver_window = XtWindow(toplevel);  /* Get Shell window ID       */
 display = XtDisplay(toplevel);         /* Get display pointer       */

/* Intern atoms that aren't predefined */

 PRIVATE_MESSAGE = XInternAtom(display, "PRIVATE_MESSAGE", False);
 RECEIVER_WINDOW = XInternAtom(display, "RECEIVER_WINDOW", False);

 XtAddEventHandler(toplevel, NoEventMask,     /* Register handler for      */
     True, MessageHandler, NULL);             /* ClientMessage events      */

 status = XChangeProperty(                    /* Put window ID in property */
     XtDisplay(toplevel),                     /* Display shared w sender   */
     RootWindowOfScreen(XtScreen(toplevel)),  /* Attach to root win        */
     RECEIVER_WINDOW,                         /* Atom = property name      */
     XA_WINDOW,                               /* Atom = property type      */
     32,                                      /* Format = 32-bit value(s)  */
     PropModeReplace,                         /* Replace any previous value*/
     (char*)&receiver_window,                 /* Window ID of Appl. Shell  */
     1);                                      /* Number of data elements   */ 

 XtAppMainLoop(context);                      /* Start event loop          */
}

/*** Application Callback Routines ***********************************/

void      
MessageHandler(w, client, event, dispatch)  /* Handle client messages*/
Widget w; 
XtPointer client; 
XEvent *event; 
Boolean *dispatch; 
{
 Atom   property;                       /* Property holding message  */
 Window window;                         /* Window owning property    */
 char   *value;                         /* Returned property value   */
 Atom   type_return;                    /* Returned property type    */
 int    format_return;                  /* Returned property format  */
 unsigned long bytes_left;              /* Returned bytes unread     */
 unsigned long length, nitems_return;   /* Requested & actual length */

 /* Ignore any other non-maskable events */

 if ((event->type != ClientMessage)
     || (event->xclient.message_type != PRIVATE_MESSAGE)) return;

 property = event->xclient.data.l[0];   /* Get data encoded in       */
 window = event->xclient.data.l[1];     /*  event structure - tells  */
 length = event->xclient.data.l[2];     /*  where to get message     */

 status = XGetWindowProperty(           /* Get message from property */
     XtDisplay(w),                      /* Display shared w sender   */
     window,                            /* Window with property      */
     property,                          /* Atom = property name      */
     0,                                 /* Offset (in 32-bit units)  */
     (length/4)+1,                      /* # of 32-bit units to read */
     True,                              /* Delete prop after reading */
     XA_STRING,                         /* Requested type            */
     &type_return,                      /* Actual type found         */
     &format_return,                    /* Returned element format   */
     &nitems_return,                    /* # of returned elements    */
     &bytes_left,                       /* Should be 0               */
     (unsigned char**)&value);          /* Returned property value   */
  
 if(status != Success){      
      /* Unsuccessful read - report failure */
      XtVaSetValues( text, XmNvalue, "Can't read message", NULL);
 }
 else {              
      /* Successful read - print message and free storage */
      XtVaSetValues( text, XmNvalue, value, NULL);
      XtFree(value);
 } 
}

/*********************************************************************/

void                                    /* Clean up and exit         */
ExitRoutine(w, client_data, call_data)
Widget w;                               /* Unused                    */
XtPointer client_data, call_data;       /* Unused                    */
{

 XDeleteProperty( XtDisplay(w),         /* Delete window property    */
     XRootWindowOfScreen(XtScreen(w)),  /*  so sender knows we       */
     RECEIVER_WINDOW);                  /*  exited                   */

 XFlush( XtDisplay(w) );                /* Make sure request is sent */
 exit(1);
}
